from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure


class Graph:

    def __init__(self, parent, file):
        self.parent = parent
        self.df = file

    def bar_chart(self, msg1, msg2) -> None:
        row_3 = self.df.loc[self.df['name'] == msg1]
        row_4 = self.df.loc[self.df['name'] == msg2]

        x_list = ['spaceflights', 'spaceflight_hours', 'spacewalks', 'spacewalk_hours']
        y_list = [
            row_3[x_list[0]].iloc[0],
            row_3[x_list[1]].iloc[0]/1000,
            row_3[x_list[2]].iloc[0],
            row_3[x_list[3]].iloc[0],
        ]
        y_list_2 = [
            row_4[x_list[0]].iloc[0],
            row_4[x_list[1]].iloc[0]/1000,
            row_4[x_list[2]].iloc[0],
            row_4[x_list[3]].iloc[0],
        ]
        fig = Figure(figsize=(1.7, 2.9), facecolor="#0b3d91")
        ax = fig.add_subplot(1, 1, 1)
        ax.set_facecolor("#0b3d91")
        ax.barh(x_list, y_list, color="#FC3D21")
        ax.set_xlim(0, 70)
        ax.invert_xaxis()

        fig_2 = Figure(figsize=(1.7, 2.9), facecolor="#0b3d91")
        ax_2 = fig_2.add_subplot(1, 1, 1)
        ax_2.set_facecolor("#0b3d91")
        ax_2.barh(x_list, y_list_2, color="#FC3D21")
        ax_2.set_xlim(0, 70)

        for item in ['xticks', 'yticks']:
            eval(f"ax.set_{item}([])")
            eval(f"ax_2.set_{item}([])")

        for item in ['top', 'bottom', 'left', 'right']:
            ax.spines[item].set_color('none')
            ax_2.spines[item].set_color('none')

        canvas = FigureCanvasTkAgg(fig, master=self.parent)
        canvas_2 = FigureCanvasTkAgg(fig_2, master=self.parent)

        canvas.get_tk_widget().place(relx=.3, rely=.581)
        canvas_2.get_tk_widget().place(relx=.55, rely=.581)

    def histogram(self, msg) -> None:
        mean = self.df[msg].mean()
        std = self.df[msg].std()
        new_data = self.df[~((self.df[msg] < mean-3*std) | (self.df[msg] > mean+3*std))]

        fig = Figure(figsize=(2.85, 1.85), facecolor="#0b3d91")
        ax = fig.add_subplot(1, 1, 1)
        ax.set_facecolor("#0b3d91")
        ax.hist(list(new_data[msg]), color="#FC3D21")
        canvas = FigureCanvasTkAgg(fig, master=self.parent)
        canvas.get_tk_widget().place(relx=.375, rely=.215)

    def heatmap(self) -> None:
        fig = Figure(figsize=(3.75, 1.75), facecolor="#0b3d91")
        ax = fig.add_subplot(1, 1, 1)
        col = self.df.loc[:, ~self.df.columns.isin(['year', 'group'])].select_dtypes(include='int')
        heatmap = ax.imshow(col.corr(), cmap='bwr')
        ax.set_xticks([n for n in range(col.shape[1])], labels=["Sf", "SfH", "Sw", "SwH"])
        ax.set_yticks([n for n in range(col.shape[1])], labels=["Sf", "SfH", "Sw", "SwH"])
        ax.tick_params(axis='both', which='major', labelsize=6)
        fig.colorbar(heatmap)
        canvas = FigureCanvasTkAgg(fig, master=self.parent)
        canvas.get_tk_widget().place(relx=.0, rely=.225)

    def boxplot(self, msg) -> None:
        mean = self.df[msg].mean()
        std = self.df[msg].std()
        new_data = self.df[~((self.df[msg] < mean-3*std) | (self.df[msg] > mean+3*std))]

        fig = Figure(figsize=(2.85, 1.85), facecolor="#0b3d91")
        ax = fig.add_subplot(1, 1, 1)
        ax.set_facecolor("#0b3d91")
        ax.boxplot(list(new_data[msg]), vert=False)
        ax.set_yticks([])
        ax.tick_params(axis='both', which='major', labelsize=6)
        canvas = FigureCanvasTkAgg(fig, master=self.parent)
        canvas.get_tk_widget().place(relx=.375, rely=.215)

    def piechart(self, msg) -> None:
        fig = Figure(figsize=(2.9, 1.9), facecolor="#0b3d91")
        total_count = self.df[msg].value_counts()
        ax = fig.add_subplot(1, 1, 1)
        ax.set_facecolor("#0b3d91")
        ax.pie(total_count.values, labels=total_count.index,
               wedgeprops={'linewidth': 1, 'edgecolor': "#0b3d91"}, autopct='%.0f%%')
        canvas = FigureCanvasTkAgg(fig, master=self.parent)
        canvas.get_tk_widget().place(relx=.64, rely=.215)
